/* 
 * File:   Morse_main.c
 * Author: peter
 *
 * Created on January 16, 2022, 3:55 PM
 */

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "Morse_header.h"


#pragma config WDTE=OFF         // We do not want the watchdog on
#pragma config CPD=OFF         // No data memory protection
#pragma config CP=OFF          // No program memory protection
#pragma config BOREN=OFF       // No brown out detection
#pragma config MCLRE=OFF         // GP3 is not MCLR
#pragma config PWRTE=OFF       // No power up timer
#pragma config FOSC=INTRCIO     // internal RC oscilator all pins I/O

#define _XTAL_FREQ 4000000      // let the compiler know that the clock frequency is 4MHz

/*
 * 
 */
// This only works because all of the Morse codes for digits are five
// characters long so we can fit in a byte (bits sent MSB first))
int get_morse_for_digit(int character) {
    int codes[10] = {0b11111000,0b01111000,0b00111000,0b00011000,0b00001000,0b00000000,0b10000000,0b11000000,0b11100000,0b11110000};
    
    return codes[character];
    }

// Morse charcacter needs to be sent bitwise MSB first '1' is a dah, '0' is a dit. dah = 3 dits
void transmit_morse_character(int character){
    int i = 5; // all digits will be 5 pulses in length
    
    for (int i = 0; i < 5; i++) {
        GPIObits.GPIO5 = 0;    // turn pin on   
        if ((character & 0b10000000) != 0) __delay_ms(dah_length); // it's a dah, do a long pulse
        else __delay_ms(dit_length); // otherwise just a short pulse
        GPIObits.GPIO5 = 1;    // turn pin off   
        __delay_ms(dit_length); // wait the interpulse space   
        character *= 2; // move along one bit
        }
    
    __delay_ms(inter_length); // wait the intercharacter space
}

int main(int argc, char** argv) {
    TRISIO = 0x00;             // Set all pins to output
    CMCON = 0x07;           // Comparitor turned off and disconnected from pins
    
    transmit_morse_character(get_morse_for_digit(4));
    transmit_morse_character(get_morse_for_digit(1));   
    transmit_morse_character(get_morse_for_digit(3));   
    transmit_morse_character(get_morse_for_digit(8));
 
 
     
    while (1) {             // and loop indefinitely             				
        GPIObits.GPIO5 = 0;    // make sure lamp is turned on when we have finished    
    }
    
    return (EXIT_SUCCESS);
}

